@@ -903,7 +903,7 @@ class TestUncertainReal(unittest.TestCase):
         self.assertTrue( un._node.independent is True )
         self.assertTrue( un._node.df == df )
         
-    def test_check_identity(self):
+    def test_check_type_identity(self):
         """
         The `reporting.is_ureal` function should be able to pick
         an uncertain real number.
@@ -920,7 +920,27 @@ class TestUncertainReal(unittest.TestCase):
 
         z1 = ucomplex(1+3j,1)
         self.assertTrue( reporting.is_ureal( z1.real ) )
+
+    def test_uid_property(self):
+        """
+        The property `uid` exposes the `uid` attribute of a `node`. 
         
+        """
+        # Elementary UN
+        un = ureal(0,1)
+        self.assertEqual(un.uid,un._node.uid)
+        self.assertEqual( un.uid, uid(un) )
+        
+        # No UID defined 
+        un2 = un * ureal(0,1)
+        self.assertTrue(un2.uid is None)
+        self.assertEqual( un2.uid, uid(un2) )
+ 
+        # Intermediate uncertain number
+        un2 = result( un * ureal(0,1) )
+        self.assertEqual( un2.uid, uid(un2) )
+        self.assertTrue(un2.uid is un2._node.uid)
+ 
     def test_strange_rounding_cases(self):
         """
         The __str__ method 
@@ -1497,8 +1517,8 @@ class TestGetCovariance(unittest.TestCase):
         self.assertTrue( equivalent( r, get_correlation_real(x1,x2) ) )
 
         # When we delete one node the other remains
-        x1_uid = x1._node.uid
-        x2_uid = x2._node.uid
+        x1_uid = x1.uid 
+        x2_uid = x2.uid 
         self.assertTrue( x1_uid in context._context._registered_leaf_nodes )
         self.assertTrue( x2_uid in context._context._registered_leaf_nodes )
         del x1
@@ -1506,7 +1526,7 @@ class TestGetCovariance(unittest.TestCase):
         self.assertTrue( x2_uid in context._context._registered_leaf_nodes )
 
         # self.assertEqual(1, len(c._registered_leaf_nodes) )
-        # self.assertTrue( x2._node.uid in c._registered_leaf_nodes )
+        # self.assertTrue( x2.uid in c._registered_leaf_nodes )
         
         # Correlation matrix is not trimmed when `x1` is destroyed
         self.assertEqual(2, len(x2._node.correlation) )
@@ -2253,9 +2273,9 @@ class TestRealEnsemble(unittest.TestCase):
         
         _ensemble = x1._node.ensemble 
         
-        self.assertTrue( x3._node.uid in _ensemble )
-        self.assertTrue( x2._node.uid in _ensemble )
-        self.assertTrue( x1._node.uid in _ensemble )
+        self.assertTrue( x3.uid in _ensemble )
+        self.assertTrue( x2.uid in _ensemble )
+        self.assertTrue( x1.uid in _ensemble )
         
         self.assertEqual(x1._node.ensemble,x2._node.ensemble)
         self.assertEqual(x1._node.ensemble,x3._node.ensemble)
